# django_autodemo/tests/test_integration.py
# django-autodemo
# Author: Rushy Panchal
# Date: April 20th, 2017
# Description: Test integration of login/logout behavior.

import re

from django.conf import settings
from django.core.urlresolvers import reverse_lazy
from django.test import TestCase
from django.contrib.auth.models import User

from django_autodemo.settings import DEMO_SETTINGS

import mock

_LOGIN_URL = settings.LOGIN_URL
_LOGOUT_URL = reverse_lazy('django-autodemo:logout')
_USER_RE = re.compile(r'^Authenticated as ([a-zA-Z0-9]+)\..*$', re.DOTALL)

class TestIntegration(TestCase):
	def get_login_user(self):
		response = self.client.get('/', follow=True)
		matches = _USER_RE.match(response.content.decode('ascii'))
		if not matches:
			self.fail('Does not match username pattern')

		username = matches.group(1)

		try:
			return response, User.objects.get(username=username)
		except User.DoesNotExist:
			return response, None

	def test_automatic_login(self):
		'''Automatic logging in should create a new user.'''
		num_users = User.objects.count()

		response, user = self.get_login_user()
		self.assertIsNotNone(user)
		self.assertEqual(response.status_code, 200)
		self.assertEqual(num_users+1, User.objects.count())

	def test_multiple_login(self):
		'''Automatic logging in multiple users should succeed.'''
		num_users = User.objects.count()

		for _ in range(3):
			response, user = self.get_login_user()
			self.assertIsNotNone(user)
			self.assertEqual(response.status_code, 200)
			self.assertEqual(num_users+1, User.objects.count())

			num_users += 1
			self.client.logout()

	def test_login_preserves_users(self):
		'''Logging in should not affect other users.'''
		User.objects.all().delete()
		User.objects.bulk_create(User(username='test-user-%d' % i) for i in range(25))

		original_users = sorted(User.objects.all(), key=lambda x: x.username)

		response, user = self.get_login_user()
		self.assertIsNotNone(user)
		self.assertEqual(response.status_code, 200)

		user.delete()
		self.assertEqual(original_users,
			sorted(User.objects.all(), key=lambda x: x.username))

	def test_logout(self):
		'''Logging out should not delete a user.'''
		num_users = User.objects.count()

		response, user = self.get_login_user()
		self.assertIsNotNone(user)
		self.assertEqual(response.status_code, 200)
		self.assertEqual(num_users+1, User.objects.count())

		self.client.get(_LOGOUT_URL, follow=True)
		self.assertEqual(num_users+1, User.objects.count())

	def test_logout_delete(self):
		'''
		Logging out a user should delete the user when that setting is enabled.
		'''
		with mock.patch.dict(DEMO_SETTINGS, {'DELETE_USER': True}):
			num_users = User.objects.count()

			response, user = self.get_login_user()
			self.assertIsNotNone(user)
			self.assertEqual(response.status_code, 200)
			self.assertEqual(num_users+1, User.objects.count())

			self.assertEqual(num_users+1, User.objects.count())

			self.client.get(_LOGOUT_URL, follow=True)
			self.assertEqual(num_users, User.objects.count())

	def test_logout_preserves_users(self):
		'''Logging in should not affect other users.'''
		with mock.patch.dict(DEMO_SETTINGS, {'DELETE_USER': True}):
			User.objects.all().delete()
			User.objects.bulk_create(User(username='test-user-%d' % i) for i in range(25))

			original_users = sorted(User.objects.all(), key=lambda x: x.username)

			response, user = self.get_login_user()
			self.assertIsNotNone(user)
			self.assertEqual(response.status_code, 200)

			self.client.get(_LOGOUT_URL, follow=True)
			self.assertEqual(original_users,
				sorted(User.objects.all(), key=lambda x: x.username))



# django_autodemo/tests/test_integration.py
# django-autodemo
# Author: Rushy Panchal
# Date: April 20th, 2017
# Description: Test signals.

import re

from django.conf import settings
from django.core.urlresolvers import reverse_lazy
from django.test import TestCase
from django.contrib.auth.models import User

from django_autodemo.settings import DEMO_SETTINGS
from django_autodemo import signals

import mock

_LOGIN_URL = settings.LOGIN_URL
_LOGOUT_URL = reverse_lazy('django-autodemo:logout')
_USER_RE = re.compile(r'^Authenticated as ([a-zA-Z0-9]+)\..*$', re.DOTALL)

class TestSignals(TestCase):
	def get_login_user(self):
		response = self.client.get('/', follow=True)
		matches = _USER_RE.match(response.content.decode('ascii'))
		if not matches:
			self.fail('Does not match username pattern')

		username = matches.group(1)

		try:
			return response, User.objects.get(username=username)
		except User.DoesNotExist:
			return response, None

	def test_login_signal(self):
		'''A login signal should be received with the correct parameters.'''
		self.user_created_signal_sent = False

		def handler(sender, request, user, **kwargs):
			self.user_created_signal_sent = True
			self.received_user = user

		signals.demo_user_created.connect(handler)

		response, self.expected_user = self.get_login_user()
		self.assertEqual(self.received_user, self.expected_user)
		self.assertTrue(self.user_created_signal_sent)

	def test_login_signal_repeated(self):
		'''A login signal should be received with the correct parameters.'''
		self.user_created_signal_count = 0

		def handler(sender, request, user, **kwargs):
			self.user_created_signal_count += 1
			self.received_user = user

		signals.demo_user_created.connect(handler)

		response, self.expected_user = self.get_login_user()
		response2 = self.client.get('/', follow=True)

		self.assertEqual(self.received_user, self.expected_user)
		self.assertEqual(self.user_created_signal_count, 1)

	def test_logout_signal(self):
		'''A logout signal should be sent when the user is logged out.'''
		self.user_logout_signal_sent = False

		def handler(sender, request, user, **kwargs):
			self.user_logout_signal_sent = True
			self.received_user = user

		signals.demo_user_logout.connect(handler)

		response, self.expected_user = self.get_login_user()
		self.client.get(_LOGOUT_URL, follow=True)
		self.assertEqual(self.received_user, self.expected_user)
		self.assertTrue(self.user_logout_signal_sent)

	def test_logout_delete_signal(self):
		'''A logout signal should be sent when the user is logged out.'''
		with mock.patch.dict(DEMO_SETTINGS, {'DELETE_USER': True}):
			self.user_logout_signal_sent = False

			def handler(sender, request, user, **kwargs):
				self.user_logout_signal_sent = True
				self.received_user = user

			signals.demo_user_logout.connect(handler)

			response, self.expected_user = self.get_login_user()
			self.client.get(_LOGOUT_URL, follow=True)
			self.assertEqual(self.received_user.username,
				self.expected_user.username)
			self.assertTrue(self.user_logout_signal_sent)



# django-autodemo/tests/settings.py
# django-autodemo
# Author: Rushy Panchal
# Date: April 19th, 2017
# Description: Test application settings.

from django.core.urlresolvers import reverse_lazy

DEBUG = True
TESTING = True

DATABASES = {
	'default': {
		'ENGINE': 'django.db.backends.sqlite3',
		'NAME': 'test.db',
		}
	}

SECRET_KEY = "BITlFxZunJXWaoiKhMAE"

ROOT_URLCONF = 'django_autodemo.tests.urls'
STATIC_URL = '/static/'

INSTALLED_APPS = (
	'django.contrib.auth',
	'django.contrib.contenttypes',
	'django.contrib.sessions',
	'django.contrib.sites',
	'django.contrib.staticfiles',
	'django.contrib.admin',

	'django_autodemo',
	)

MIDDLEWARE_CLASSES = (
  'django.middleware.security.SecurityMiddleware',
  'django.contrib.sessions.middleware.SessionMiddleware',
  'django.middleware.common.CommonMiddleware',
  'django.middleware.csrf.CsrfViewMiddleware',
  'django.contrib.auth.middleware.AuthenticationMiddleware',
  'django.contrib.auth.middleware.SessionAuthenticationMiddleware',
  'django.contrib.messages.middleware.MessageMiddleware',
  'django.middleware.clickjacking.XFrameOptionsMiddleware',
)

AUTHENTICATION_BACKENDS = (
	'django.contrib.auth.backends.ModelBackend',
	)

DEMO_SETTINGS = {
	'ENABLED': True,
	}
LOGIN_URL = reverse_lazy('django-autodemo:login')



# django_autodemo/apps.py
# django_autodemo
# Author: Rushy Panchal
# Date: April 20th, 2017
# Description: App configuration for Django.

from django.apps import AppConfig

class WsseConfig(AppConfig):
	name = 'django_autodemo'



# django_autodemo/urls.py
# django-autodemo
# Author: Rushy Panchal
# Date: April 19th, 2017
# Description: Authentication urls for django-autodemo.

from django.conf.urls import url
from django_autodemo.views import login_view, logout_view

app_name = 'django-autodemo'
urlpatterns = [
	url(r'^login$', login_view, name='login'),
	url(r'^logout$', logout_view, name='logout')
	]



# django-autodemo/tests/views.py
# django-autodemo
# Author: Rushy Panchal
# Date: April 19th ,2017
# Description: Test views.

from django.http import HttpResponse
from django.contrib.auth.decorators import login_required
from django.core.urlresolvers import reverse

@login_required
def demo_view(request):
	return HttpResponse('''Authenticated as {u}.
		<a href="{logout}?next=/">Logout.</a>'''.format(
			logout=reverse('django-autodemo:logout'),
			u=request.user.username)
			)



# django-autodemo/tests/urls.py
# django-autodemo
# Author: Rushy Panchal
# Date: April 19th ,2017
# Description: Test URL configuration.

from django.conf.urls import url, include
from django_autodemo.tests import views

urlpatterns = [
	url(r'^$', views.demo_view, name='demo'),
	url(r'^demo/', include('django_autodemo.urls'))
	]



# django_autodemo/signals.py
# django-autodemo
# Author: Rushy Panchal
# Date: April 20th, 2017
# Description: Signals for django-autodemo.

from django.dispatch import Signal

demo_user_created = Signal(providing_args=('request', 'user'))
demo_user_logout = Signal(providing_args=('request', 'user'))



# setup.py
# django-autodemo
# Author: Rushy Panchal
# Date: April 20th, 2017
# Description: Setuptools configuration.

from setuptools import setup, find_packages

setup(
	name = 'django-autodemo',
	packages = find_packages(),
	version = '0.1.0',
	description = 'Demo mode to allow arbitrary authentication on website.',
	author = 'Rushy Panchal',
	author_email = 'rpanchal@princeton.edu',
	url = 'https://github.com/panchr/django-autodemo',
	keywords = ['django', 'authentication', 'demo'],
	license = 'LGPLv3',
	classifiers = [
		'Programming Language :: Python :: 2',
		'Programming Language :: Python :: 2.7',
		'Programming Language :: Python :: 3',
		'Programming Language :: Python :: 3.4',
		'Programming Language :: Python :: 3.5',
		'Topic :: Internet :: WWW/HTTP :: WSGI :: Middleware',
		'Topic :: Software Development :: Libraries :: Python Modules',
		'Intended Audience :: Developers',
		'Operating System :: OS Independent',
		'License :: OSI Approved :: GNU Lesser General Public License v3 (LGPLv3)',
		],
)


# django_autodemo/views.py
# django-autodemo
# Author: Rushy Panchal
# Date: April 19th, 2017
# Description: Authentication views for django-autodemo.

from django.conf import settings
from django.http import HttpResponseRedirect
from django.contrib.auth import (authenticate, login,
	REDIRECT_FIELD_NAME, logout)
from django.contrib.auth.models import User
from django.utils.crypto import get_random_string
from django.db import IntegrityError

from django_autodemo import signals
from django_autodemo.settings import DEMO_SETTINGS

def login_view(request):
	'''
	On an unauthenticated request, automatically generate a user and log that
	user in. This allows for demo users to be made automatically.
	'''
	next_page = request.GET.get(REDIRECT_FIELD_NAME)
	if request.user.is_authenticated():
		# Redirect user if already authenticated.
		return HttpResponseRedirect(next_page)

	if DEMO_SETTINGS['ENABLED']:
		user = None
		attempts = 0

		# Attempt to create user with random username _MAX_ATTEMPTS times.
		while user is None and attempts < DEMO_SETTINGS['MAX_USER_ATTEMPTS']:
			try:
				username=get_random_string(DEMO_SETTINGS['USERNAME_LENGTH'])
				user = User.objects.create(username=username)
			except IntegrityError:
				attempts += 1

	if user:
		user.backend = settings.AUTHENTICATION_BACKENDS[0]
		login(request, user)
		signals.demo_user_created.send(sender=login_view, request=request,
			user=user)

	return HttpResponseRedirect(next_page)

def logout_view(request):
	'''
	On an authenticated request, logout and (optionally) delete the user.
	'''
	next_page = request.GET.get(REDIRECT_FIELD_NAME)
	if not request.user.is_authenticated():
		# Redirect user if not currently authenticated.
		return HttpResponseRedirect(next_page)

	if DEMO_SETTINGS['ENABLED']:
		user = request.user
		logout(request)
		signals.demo_user_logout.send(sender=login_view, request=request,
			user=user)

		if DEMO_SETTINGS['DELETE_USER']:
			user.delete()

	return HttpResponseRedirect(next_page)



# django_autodemo/settings.py
# django-autodemo
# Author: Rushy Panchal
# Date: April 20th, 2017
# Description: Automatically load settings

from django.conf import settings

DEFAULT_SETTINGS = {
	'ENABLED': False,
	'MAX_USER_ATTEMPTS': 10,
	'USERNAME_LENGTH': 16,
	'DELETE_USER': False,
	}

DEMO_SETTINGS = DEFAULT_SETTINGS.copy()
DEMO_SETTINGS.update(getattr(settings, 'DEMO_SETTINGS', {}))
